#
# Copyright (c) 2013 - 2016 MasterCard International Incorporated
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, are 
# permitted provided that the following conditions are met:
# 
# Redistributions of source code must retain the above copyright notice, this list of 
# conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice, this list of 
# conditions and the following disclaimer in the documentation and/or other materials 
# provided with the distribution.
# Neither the name of the MasterCard International Incorporated nor the names of its 
# contributors may be used to endorse or promote products derived from this software 
# without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
# IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
# SUCH DAMAGE.
#

require 'simplify/paymentsapi'

module Simplify

# A Authorization object.
#
class Authorization < Hash

    # Authentication object used to access the API (See Simplify::Authentication for details)
    attr_accessor :authentication

    # Returns the public key used when accessing this object. <b>Deprecated: please use 'authentication' instead.</b>
    def public_key
        return self.authentication.public_key
    end

    # Sets the public key used when accessing this object. <b>Deprecated: please use 'authentication' instead.</b>
    def public_key=(k)
        return self.authentication.public_key = k
    end

    # Returns the private key used when accessing this object. <b>Deprecated: please use 'authentication' instead.</b>
    def private_key
        return self.authentication.private_key
    end

    # Sets the private key used when accessing this object. <b>Deprecated: please use 'authentication' instead.</b>
    def private_key=(k)
        return self.authentication.private_key = k
    end


    # Creates an Authorization object
    #
    # parms:: a hash of parameters; valid keys are:
    # * <code>amount</code>    Amount of the payment (in the smallest unit of your currency). Example: 100 = $1.00USD <b>required </b>
    # * <code>card => addressCity</code>    City of the cardholder. [max length: 50, min length: 2]
    # * <code>card => addressCountry</code>    Country code (ISO-3166-1-alpha-2 code) of residence of the cardholder. [max length: 2, min length: 2]
    # * <code>card => addressLine1</code>    Address of the cardholder. [max length: 255]
    # * <code>card => addressLine2</code>    Address of the cardholder if needed. [max length: 255]
    # * <code>card => addressState</code>    State of residence of the cardholder. For the US, this is a 2-digit USPS code. [max length: 255]
    # * <code>card => addressZip</code>    Postal code of the cardholder. The postal code size is between 5 and 9 characters in length and only contains numbers or letters. [max length: 32]
    # * <code>card => cvc</code>    CVC security code of the card. This is the code on the back of the card. Example: 123
    # * <code>card => expMonth</code>    Expiration month of the card. Format is MM. Example: January = 01 [min value: 1, max value: 12] <b>required </b>
    # * <code>card => expYear</code>    Expiration year of the card. Format is YY. Example: 2013 = 13 [min value: 0, max value: 99] <b>required </b>
    # * <code>card => name</code>    Name as it appears on the card. [max length: 50, min length: 2]
    # * <code>card => number</code>    Card number as it appears on the card. [max length: 19, min length: 13] <b>required </b>
    # * <code>currency</code>    Currency code (ISO-4217) for the transaction. Must match the currency associated with your account. [default: USD] <b>required </b>
    # * <code>customer</code>    ID of customer. If specified, card on file of customer will be used.
    # * <code>description</code>    Free form text field to be used as a description of the payment. This field is echoed back with the payment on any find or list operations. [max length: 1024]
    # * <code>order => commodityCode</code>    Standard classification code for products and services. [max length: 5]
    # * <code>order => customer</code>    ID of the customer associated with the order.
    # * <code>order => customerEmail</code>    Customer email address.
    # * <code>order => customerName</code>    Customer name.
    # * <code>order => customerNote</code>    Additional notes provided by the customer. [max length: 255]
    # * <code>order => customerReference</code>    A merchant reference for the customer.
    # * <code>order => items => amount</code>    Cost of the item.
    # * <code>order => items => description</code>    Description of the item.
    # * <code>order => items => name</code>    Item name.
    # * <code>order => items => product</code>    Product information associated with the item.
    # * <code>order => items => quantity</code>    Quantity of the item contained in the order [min value: 1, max value: 999999, default: 1] <b>required </b>
    # * <code>order => items => reference</code>    A merchant reference for the item. [max length: 255]
    # * <code>order => items => tax</code>    Taxes associated with the item.
    # * <code>order => merchantNote</code>    Additional notes provided by the merchant. [max length: 255]
    # * <code>order => payment</code>    ID of the payment associated with the order.
    # * <code>order => reference</code>    A merchant reference for the order. [max length: 255]
    # * <code>order => shippingAddress => city</code>    City, town, or municipality. [max length: 255, min length: 2]
    # * <code>order => shippingAddress => country</code>    2-character country code. [max length: 2, min length: 2]
    # * <code>order => shippingAddress => line1</code>    Street address. [max length: 255]
    # * <code>order => shippingAddress => line2</code>    (Opt) Street address continued. [max length: 255]
    # * <code>order => shippingAddress => name</code>    Name of the entity being shipped to. [max length: 255]
    # * <code>order => shippingAddress => state</code>    State or province. [max length: 255]
    # * <code>order => shippingAddress => zip</code>    Postal code. [max length: 32]
    # * <code>order => shippingFromAddress => city</code>    City, town, or municipality. [max length: 255, min length: 2]
    # * <code>order => shippingFromAddress => country</code>    2-character country code. [max length: 2, min length: 2]
    # * <code>order => shippingFromAddress => line1</code>    Street address. [max length: 255]
    # * <code>order => shippingFromAddress => line2</code>    (Opt) Street address continued. [max length: 255]
    # * <code>order => shippingFromAddress => name</code>    Name of the entity performing the shipping. [max length: 255]
    # * <code>order => shippingFromAddress => state</code>    State or province. [max length: 255]
    # * <code>order => shippingFromAddress => zip</code>    Postal code. [max length: 32]
    # * <code>order => shippingName</code>    Name of the entity being shipped to.
    # * <code>order => source</code>    Order source. [default: WEB] <b>required </b>
    # * <code>order => status</code>    Status of the order. [default: INCOMPLETE] <b>required </b>
    # * <code>reference</code>    Custom reference field to be used with outside systems.
    # * <code>replayId</code>    An identifier that can be sent to uniquely identify a payment request to facilitate retries due to I/O related issues. This identifier must be unique for your account (sandbox or live) across all of your payments. If supplied, we will check for a payment on your account that matches this identifier, and if one is found we will attempt to return an identical response of the original request. [max length: 50, min length: 1]
    # * <code>statementDescription => name</code>    Merchant name <b>required </b>
    # * <code>statementDescription => phoneNumber</code>    Merchant contact phone number.
    # * <code>token</code>    If specified, card associated with card token will be used. [max length: 255]
    # auth:: Authentication information used for the API call.  If no value is passed the global keys Simplify::public_key and Simplify::private_key are used.  For backwards compatibility the public and private keys may be passed instead of the authentication object.
    # Returns a Authorization object.
    def self.create(parms, *auth)

        auth_obj = Simplify::PaymentsApi.create_auth_object(auth)
        h = Simplify::PaymentsApi.execute("authorization", 'create', parms, auth_obj)
        obj = Authorization.new()
        obj.authentication = auth_obj
        obj = obj.merge!(h)
        obj
    end

    # Delete this object
    def delete()
        h = Simplify::PaymentsApi.execute("authorization", 'delete', self, self.authentication)
        self.merge!(h)
        self
    end

    # Retrieve Authorization objects.
    # criteria:: a hash of parameters; valid keys are: 
    # * <code>filter</code>    <table class="filter_list"><tr><td>filter.id</td><td>Filter by the Authorization Id</td></tr><tr><td>filter.replayId</td><td>Filter by the compoundReplayId</td></tr><tr><td>filter.last4</td><td>Filter by the card number (last 4 digits)</td></tr><tr><td>filter.amount</td><td>Filter by the transaction amount (in the smallest unit of your currency)</td></tr><tr><td>filter.text</td><td>Filter by the description of the authorization</td></tr><tr><td>filter.amountMin & filter.amountMax</td><td>The filter amountMin must be used with amountMax to find authorizations with authorization values between the min and max</td></tr><tr><td>filter.dateCreatedMin<sup>*</sup></td><td>Filter by the minimum created date you are searching for - Date in UTC millis</td></tr><tr><td>filter.dateCreatedMax<sup>*</sup></td><td>Filter by the maximum created date you are searching for - Date in UTC millis</td></tr><tr><td>filter.deposit</td><td>Filter by the deposit id</td></tr><tr><td>filter.customer</td><td>Filter using the Id of the customer to find the authorizations for that customer</td></tr><tr><td>filter.status</td><td>Filter by the authorization status text</td></tr><tr><td>filter.authCode</td><td>Filter by the authorization code (Not the authorization ID)</td></tr><tr><td>filter.q</td><td>You can use this to filter by the ID, the authCode or the amount of the authorization</td></tr></table><br><sup>*</sup>Use dateCreatedMin with dateCreatedMax in the same filter if you want to search between two created dates 
    # * <code>max</code>    Allows up to a max of 50 list items to return. [min value: 0, max value: 50, default: 20] 
    # * <code>offset</code>    Used in pagination of the list. This is the start offset of the page. [min value: 0, default: 0] 
    # * <code>sorting</code>    Allows for ascending or descending sorting of the list. The value maps properties to the sort direction (either <code>asc</code> for ascending or <code>desc</code> for descending).  Sortable properties are: <code> dateCreated</code><code> amount</code><code> id</code><code> description</code><code> paymentDate</code>.
    # auth:: Authentication information used for the API call.  If no value is passed the global keys Simplify::public_key and Simplify::private_key are used.  For backwards compatibility the public and private keys may be passed instead of the authentication object.
    # Returns an object where the <code>list</code> property contains the list of Authorization objects and the <code>total</code>
    # property contains the total number of Authorization objects available for the given criteria.
    def self.list(criteria = nil, *auth)

        auth_obj = Simplify::PaymentsApi.create_auth_object(auth)
        h = Simplify::PaymentsApi.execute("authorization", 'list', criteria, auth_obj)
        obj = Authorization.new()
        obj.authentication = auth_obj
        obj = obj.merge!(h)
        obj

    end

    # Retrieve a Authorization object from the API
    #
    # id:: ID of object to retrieve
    # auth:: Authentication information used for the API call.  If no value is passed the global keys Simplify::public_key and Simplify::private_key are used.  For backwards compatibility the public and private keys may be passed instead of the authentication object.
    # Returns a Authorization object.
    def self.find(id, *auth)

        auth_obj = Simplify::PaymentsApi.create_auth_object(auth)
        h = Simplify::PaymentsApi.execute("authorization", 'show', {"id" => id}, auth_obj)
        obj = Authorization.new()
        obj.authentication = auth_obj
        obj = obj.merge!(h)
        obj
    end

end
end
