#
# Copyright (c) 2013 - 2016 MasterCard International Incorporated
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, are 
# permitted provided that the following conditions are met:
# 
# Redistributions of source code must retain the above copyright notice, this list of 
# conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice, this list of 
# conditions and the following disclaimer in the documentation and/or other materials 
# provided with the distribution.
# Neither the name of the MasterCard International Incorporated nor the names of its 
# contributors may be used to endorse or promote products derived from this software 
# without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
# IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
# SUCH DAMAGE.
#

require 'simplify/paymentsapi'

module Simplify

# A FraudCheck object.
#
class FraudCheck < Hash

    # Authentication object used to access the API (See Simplify::Authentication for details)
    attr_accessor :authentication

    # Returns the public key used when accessing this object. <b>Deprecated: please use 'authentication' instead.</b>
    def public_key
        return self.authentication.public_key
    end

    # Sets the public key used when accessing this object. <b>Deprecated: please use 'authentication' instead.</b>
    def public_key=(k)
        return self.authentication.public_key = k
    end

    # Returns the private key used when accessing this object. <b>Deprecated: please use 'authentication' instead.</b>
    def private_key
        return self.authentication.private_key
    end

    # Sets the private key used when accessing this object. <b>Deprecated: please use 'authentication' instead.</b>
    def private_key=(k)
        return self.authentication.private_key = k
    end


    # Creates an FraudCheck object
    #
    # parms:: a hash of parameters; valid keys are:
    # * <code>amount</code>    Amount of the transaction to be checked for fraud (in the smallest unit of your currency). Example: 100 = $1.00USD. This field is required if using “full” or “advanced” mode.
    # * <code>card => addressCity</code>    City of the cardholder. [max length: 50, min length: 2]
    # * <code>card => addressCountry</code>    Country code (ISO-3166-1-alpha-2 code) of residence of the cardholder. [max length: 2, min length: 2]
    # * <code>card => addressLine1</code>    Address of the cardholder. [max length: 255]
    # * <code>card => addressLine2</code>    Address of the cardholder if needed. [max length: 255]
    # * <code>card => addressState</code>    State of residence of the cardholder. For the US, this is a 2-digit USPS code. [max length: 255]
    # * <code>card => addressZip</code>    Postal code of the cardholder. The postal code size is between 5 and 9 characters in length and only contains numbers or letters. [max length: 32]
    # * <code>card => cvc</code>    CVC security code of the card. This is the code on the back of the card. Example: 123
    # * <code>card => expMonth</code>    Expiration month of the card. Format is MM. Example: January = 01 [min value: 1, max value: 12] <b>required </b>
    # * <code>card => expYear</code>    Expiration year of the card. Format is YY. Example: 2013 = 13 [min value: 0, max value: 99] <b>required </b>
    # * <code>card => name</code>    Name as it appears on the card. [max length: 50, min length: 2]
    # * <code>card => number</code>    Card number as it appears on the card. [max length: 19, min length: 13] <b>required </b>
    # * <code>currency</code>    Currency code (ISO-4217) for the transaction to be checked for fraud. This field is required if using “full” or “advanced” mode.
    # * <code>description</code>    - Description of the fraud check. [max length: 255]
    # * <code>ipAddress</code>    IP Address of the customer for which the fraud check is to be done. [max length: 45]
    # * <code>mode</code>    Fraud check mode.  “simple” only does an AVS and CVC check; “advanced” does a complete fraud check, running the input against the set up rules. [valid values: simple, advanced, full] <b>required </b>
    # * <code>sessionId</code>    Session ID used during data collection. [max length: 255]
    # * <code>token</code>    Card token token representing card details for the card to be checked. [max length: 255]
    # auth:: Authentication information used for the API call.  If no value is passed the global keys Simplify::public_key and Simplify::private_key are used.  For backwards compatibility the public and private keys may be passed instead of the authentication object.
    # Returns a FraudCheck object.
    def self.create(parms, *auth)

        auth_obj = Simplify::PaymentsApi.create_auth_object(auth)
        h = Simplify::PaymentsApi.execute("fraudCheck", 'create', parms, auth_obj)
        obj = FraudCheck.new()
        obj.authentication = auth_obj
        obj = obj.merge!(h)
        obj
    end

    # Retrieve FraudCheck objects.
    # criteria:: a hash of parameters; valid keys are: 
    # * <code>filter</code>    Filters to apply to the list. 
    # * <code>max</code>    Allows up to a max of 50 list items to return. [min value: 0, max value: 50, default: 20] 
    # * <code>offset</code>    Used in paging of the list.  This is the start offset of the page. [min value: 0, default: 0] 
    # * <code>sorting</code>    Allows for ascending or descending sorting of the list. The value maps properties to the sort direction (either <code>asc</code> for ascending or <code>desc</code> for descending).  Sortable properties are: <code> amount</code><code> dateCreated</code><code> fraudResult</code>.
    # auth:: Authentication information used for the API call.  If no value is passed the global keys Simplify::public_key and Simplify::private_key are used.  For backwards compatibility the public and private keys may be passed instead of the authentication object.
    # Returns an object where the <code>list</code> property contains the list of FraudCheck objects and the <code>total</code>
    # property contains the total number of FraudCheck objects available for the given criteria.
    def self.list(criteria = nil, *auth)

        auth_obj = Simplify::PaymentsApi.create_auth_object(auth)
        h = Simplify::PaymentsApi.execute("fraudCheck", 'list', criteria, auth_obj)
        obj = FraudCheck.new()
        obj.authentication = auth_obj
        obj = obj.merge!(h)
        obj

    end

    # Retrieve a FraudCheck object from the API
    #
    # id:: ID of object to retrieve
    # auth:: Authentication information used for the API call.  If no value is passed the global keys Simplify::public_key and Simplify::private_key are used.  For backwards compatibility the public and private keys may be passed instead of the authentication object.
    # Returns a FraudCheck object.
    def self.find(id, *auth)

        auth_obj = Simplify::PaymentsApi.create_auth_object(auth)
        h = Simplify::PaymentsApi.execute("fraudCheck", 'show', {"id" => id}, auth_obj)
        obj = FraudCheck.new()
        obj.authentication = auth_obj
        obj = obj.merge!(h)
        obj
    end

    # Updates this object
    #
    # The properties that can be updated:
    # * <code>integratorAuthCode</code> Authorization code for the transaction. [max length: 255]
# * <code>integratorAvsAddressResponse</code> AVS address response. [max length: 255]
# * <code>integratorAvsZipResponse</code> AVS zip response. [max length: 255]
# * <code>integratorCvcResponse</code> CVC response. [max length: 255]
# * <code>integratorDeclineReason</code> Reason for the decline if applicable. [max length: 255]
# * <code>integratorTransactionRef</code> Reference id for the transaction. [max length: 255] <b>(required)</b>
# * <code>integratorTransactionStatus</code> Status of the transaction, valid values are "APPROVED", "DECLINED", "SETTLED", "REFUNDED" or "VOIDED".
    def update()
          h = Simplify::PaymentsApi.execute("fraudCheck", 'update', self, self.authentication)
          self.merge!(h)
          self
    end

end
end
